DEFAULT_CONFIG = {
    'starting_balance': 100000,
    'data_provider': 'yahoo',
    'candle_aggregation': '1d',
    'ticker': 'AAPL',
    'start_date': '2023-01-01',
    'end_date': '2023-06-01',
    'shares': 10,
    'print_tradehistory': False,
    'print_pnl': True,
    'print_balance': True,
    'print_buypower': True,
    'print_unrealizedpnl': True,
    'print_timecomplexity': True,
    'chart': True
}
